#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class VPN : Indicator
	{	
		private VOLMA VolumeMA; //MAV
		private ATR ATR1;
		private SUM SUM1; //sum for VMP
		private SUM SUM2; //sum for VMN
		private EMA EMAVPN;
		private SMA MAVPN;
		
		private double MF = -1;
		private double MC = -1;
		
		private Series<double> VMP;
		private Series<double> VMN;
		private Series<double> VPNSeries;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"The VPN indicator as published in April 2021 Technical Analysis of Stocks and Commodities article “The Volume Positive Negative (VPN) Indicator Detecting High-Volume Breakouts” by Markos Katsanos.";
				Name										= "VPN";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				Period										= 30;
				Smooth										= 3;
				VPNCRIT										= 10;
				MAB											= 30;
				
				AddPlot(Brushes.LimeGreen, "AboveCrit");
				AddPlot(Brushes.Red, "BelowCrit");
				AddPlot(Brushes.Orange, "MAVPNPlot");	
			}
			else if (State == State.Configure)
			{
				Plots[0].Min = VPNCRIT;
				Plots[1].Max = VPNCRIT;
				AddLine(Brushes.RoyalBlue, VPNCRIT, "VPNCRITLINE");
			}
			else if (State == State.DataLoaded)
			{
				VMP 	  = new Series<double>(BarsArray[0], MaximumBarsLookBack.Infinite);
				VMN 	  = new Series<double>(BarsArray[0], MaximumBarsLookBack.Infinite);
				VPNSeries = new Series<double>(BarsArray[0], MaximumBarsLookBack.Infinite);
				
				VolumeMA  = VOLMA(Period);
				ATR1 	  = ATR(Period);
				SUM1 	  = SUM(VMP, Period);
				SUM2 	  = SUM(VMN, Period);
				EMAVPN 	  = EMA(VPNSeries, Smooth);
				MAVPN 	  = SMA(EMAVPN, MAB);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < Period || CurrentBar < MAB)
				return;
			
			double MF = Typical[0] - Typical[1];
			double MC = ATR1[0] * 0.1;
			
			VMP[0] = MF > MC  ? Volume[0] : 0;	
			VMN[0] = MF < -MC ?	Volume[0] : 0;
			VPNSeries[0] = (SUM1[0] - SUM2[0]) / (VolumeMA[0] == 0 ? 1 : VolumeMA[0]) / Period * 100;
			
			AboveCrit[0] = EMAVPN[0];
			BelowCrit[0] = EMAVPN[0];
			MAVPNPlot[0] = MAVPN[0];
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Order=1, GroupName="Parameters")]
		public int Period
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Smoothing Constant", Order=2, GroupName="Parameters")]
		public int Smooth
		{ get; set; }

		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="VPN Crit", Order=3, GroupName="Parameters")]
		public double VPNCRIT
		{ get; set; }

		[NinjaScriptProperty]
		[Range(10, int.MaxValue)]
		[Display(Name="Moving Average Bars", Description="MA Bars", Order=4, GroupName="Parameters")]
		public int MAB
		{ get; set; }
		
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> AboveCrit
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> BelowCrit
		{
			get { return Values[1]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> MAVPNPlot
		{
			get { return Values[2]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private VPN[] cacheVPN;
		public VPN VPN(int period, int smooth, double vPNCRIT, int mAB)
		{
			return VPN(Input, period, smooth, vPNCRIT, mAB);
		}

		public VPN VPN(ISeries<double> input, int period, int smooth, double vPNCRIT, int mAB)
		{
			if (cacheVPN != null)
				for (int idx = 0; idx < cacheVPN.Length; idx++)
					if (cacheVPN[idx] != null && cacheVPN[idx].Period == period && cacheVPN[idx].Smooth == smooth && cacheVPN[idx].VPNCRIT == vPNCRIT && cacheVPN[idx].MAB == mAB && cacheVPN[idx].EqualsInput(input))
						return cacheVPN[idx];
			return CacheIndicator<VPN>(new VPN(){ Period = period, Smooth = smooth, VPNCRIT = vPNCRIT, MAB = mAB }, input, ref cacheVPN);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.VPN VPN(int period, int smooth, double vPNCRIT, int mAB)
		{
			return indicator.VPN(Input, period, smooth, vPNCRIT, mAB);
		}

		public Indicators.VPN VPN(ISeries<double> input , int period, int smooth, double vPNCRIT, int mAB)
		{
			return indicator.VPN(input, period, smooth, vPNCRIT, mAB);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.VPN VPN(int period, int smooth, double vPNCRIT, int mAB)
		{
			return indicator.VPN(Input, period, smooth, vPNCRIT, mAB);
		}

		public Indicators.VPN VPN(ISeries<double> input , int period, int smooth, double vPNCRIT, int mAB)
		{
			return indicator.VPN(input, period, smooth, vPNCRIT, mAB);
		}
	}
}

#endregion
